module datastore;

import std.conv;
import vibe.d;

/// This class handles 
/// the postage of new message
/// to the drop
/// FIXME: add sqlite backend for message 
///        storage so messages survive restarts
class Backend
{
    private struct Message
    {
        string recipient;
        string payload;
    }

    private Message[] p_message;

    private string serializeMessage(Message msg)
    {
        immutable string msg_template = `
        {
            "recipient": "%s",
            "payload": "%s"
        }
        `;

        import std.format : format;
        return format(msg_template, msg.recipient, msg.payload);
    }

    /// This function gets all the posted
    /// messages as a json array
    @path("/messages")
    @safe
    Json getMessages(HTTPServerResponse res,)
    {
        return serializeToJson(p_message);
    }

    /// This function is
    /// used to add a new 
    /// message to the drop-point
    /// post data like so
    /// {
    ///     "recipient": "base64 encoded publickey",
    ///     "sender": "base64 encoded publickey",
    ///     "payload": "base64 encoded payload data",
    ///     "nonce": "base64 encoded nonce"
    /// }
    @path("/add-message")
    @method(HTTPMethod.POST)
    @safe
    void addMessage(
        string recipient,
        string payload,
        HTTPServerResponse res,
    )
    {
        logInfo(text("Backend: POST /add-message for : ", recipient));

        auto m = Message(recipient, payload);

        this.p_message ~= m;

        res.writeBody("");
    }

    @path("/getmessages")
    @method(HTTPMethod.POST)
    @safe
    Json getMessageForPubkeys(string pubkey)
    {
        logInfo(text("Backend: POST /getmessages for : ", pubkey));

        Message[] user_messages;

        foreach (Message key; p_message)
        {
            if (key.recipient == pubkey)
            {
                user_messages ~= key;
            }
        }

        return serializeToJson(user_messages);
    }
}